procedure RTFDDGenerate (pTempDoc, pDocName, pDirName, pFileName, inout pTopicNumber, pKeyWord) \
                local (pSectionNumber, \
                       pTopicSize, \
                       pSectionSize, \
                       pMaxTopicSize, \
                       pPos, \
                       pC1, \
                       pC2, \
                       pFill, \
                       pFill2, \
                       pChangeMade, \
                       pRefObjName)
                       
   SET SAVE
   set escapechar on
   set information off
   set message window on
   set delimiter "\01"
   set headings off
   set checksets off
   set output trim on

   let pMaxTopicSize = 25000

   let <PgmName> = pTempDoc
   find #<PgmName> -> HelpDocSet

   set output RTFOutput append
   let pTopicSize = 0 \
       pSectionNumber = 0

   %% output topic header for first topic
   let <RTFTitle> = $concat($trim(pDocName), \
                            "[", \
                            $trim(pDirName), \
                            "]" ) \
       <RTFBrowseString> = $concat($trim(pFileName), ":", $trim(pTopicNumber+100))  \
       <TopicHeading> = $concat ($trim(pDocName), \
                                 "[", \
                                 $trim(pDirName), \
                                 "]" ) 

   let <RTFContextString> = $concat($trim($translate(pDocName,"$","C_")), \
                               "_", $trim(pTopicNumber)) \
       <RTFKeywords> = $concat($trim(pDocName), ";", pKeyWord)

   generate TopicHeader

   %% find objects ref'd by this one
   find ObjList Dep2ObjList Dependencies DDep2ObjList DependentObjs \
      where ObjList.ObjectName=pDocName and ObjList.DirName=pDirName and \
            Dependencies.Flag in ("O", "I") keep DependentObjs
   find CURRENT (DependentObjs -> ObjList)
   find CURRENT intersect $HelpGenSet
   find CURRENT \
      where ObjectType in ("Document", \
                           "EntitySet", \
                           "Set", \
                           "Variable", \
                           "Form", \
                           "Display", \
                           "Menu", \
                           "Window", \
                           "Relationship", \
                           "Role") \
      -> $HotSpotSet

   while
      let pSectionNumber = pSectionNumber + 1
      %% get next bunch of lines and process
      set escapechar off
      set specialscan off
      change 25 $ZOM.fHelpLine from HelpDocSet -> $HelpLineSet
      set escapechar on
      set specialscan on

      %% change all tabs to 2 spaces
      %% change all backslashes to doubles
      %% change all curly brackets to backslash curly
      change all $HelpLineSet let \
         HelpLine = $translate( \
                       $translate ( \
                          $translate ( \
                             $translate(HelpLine, "\09", "  "), \
                             "\\", "\\\\"), \
                          "{", "\\{"), \
                       "}", "\\}")

      %% insert hot spots

      top $HotSpotSet
      if $lastmember("$HotSpotSet") <= 0
         goto StartOutput
      endif
      while
         set currentset $HotSpotSet
         let pRefObjName = ObjList.ObjectName \
             pChangeMade = $false \
             pFill = $tochar(" ", 18)  \
             pFill = $fill( pFill, 1, $length($trim(pRefObjName)), "\02") \
             pFill2 = $tochar(" ",18) \
             pFill2 = $fill(pFill2, 1, $length($trim(pRefObjName)), "\03")

         while
            change all $HelpLineSet where \
               ((let pPos = $Position($toalpha(HelpLine,-1),pRefObjName)), \
                (let pC1 = $substring(HelpLine,pPos-1,1)), \
                (let pC2 = $substring(HelpLine,pPos+$length(pRefObjName),1)), \
                pPos) > 0 \
               let HelpLine = {$concat($substring(HelpLine,1,pPos-1), \
                                      pFill, \
                                      $substring(HelpLine,pPos+$length(pRefObjName), 256)) \
                                   where \
                                      $iszimname($trim(pC1)) = $true or \
                                      $iszimname($trim(pC2)) = $true or \
                                      $isdigit($trim(pC1)) = $true or \
                                      $isdigit($trim(pC2)) = $true, \
                               $concat($substring(HelpLine,1,pPos-1), \
                                      "{\\uldb ", \
                                      pFill, \
                                      "}{\\v ", \
                                      pFill2, \
                                      "_1}", \
                                      $substring(HelpLine,pPos+$length(pFill), 256))}
            if $membercount > 0
               let pChangeMade = $true
            else
               break
            endif
         endwhile

         if pChangeMade = $true
            change all $HelpLineSet let \
               HelpLine = $translate( \
                             $translate (HelpLine, pFill2, \
                                $translate(pRefObjName,"$","C_")), \
                                pFill, \
                                pRefObjName)
         endif


         if $CurrentMember("$HotSpotSet") = $LastMember("$HotSpotSet")
            break
         else
            next $HotSpotSet
         endif
      endwhile

StartOutput:
      %% Prepare to output to RTF file
      let pSectionSize = 0
      compute all $HelpLineSet eval (let pSectionSize = pSectionSize + \
                    6 + $length($trim(HelpLine)))

      if pTopicSize + pSectionSize > pMaxTopicSize
         %% First Close off the current topic
         generate TopicFooter
         let pTopicNumber = pTopicNumber + 1

         %% Then gen topic header
         let <RTFTitle> = $concat($trim(pDocName), "/", $trim(pTopicNumber)) \
             <RTFBrowseString> = $concat($trim(pFileName), ":", $trim(pTopicNumber+100))  \
             <TopicHeading> = $concat($trim(pDocName), "/", $trim(pTopicNumber)) 

         let <RTFContextString> = "" \
             <RTFKeyWords> = ""

         generate TopicHeader
         let pTopicSize = 0
         let pSectionNumber = 1
      endif

<<
      if pSectionNumber = 1
         %% Don't insert LINE command before very first line
         next $HelpLineSet
      endif
>>
      change all $HelpLineSet \
         let $ZOM.fHelpLine.HelpLine = \
            $concat("\\line ", $ZOM.fHelpLine.HelpLine)
      top $HelpLineSet

      list all $HelpLineSet 
      let pTopicSize = pTopicSize + pSectionSize

      if $CurrentMember ("HelpDocSet") + 25 <= $LastMember("HelpDocSet")
         next 25 HelpDocSet
      else
         break
      endif
   endwhile

   set output terminal

   SET RESTORE
   set escapechar on
endprocedure
   
