%%%   List fields of an Object
%%% Author: Livio Taufer
%%% Syntax:
%%%         $lsflds <objectname> [IND | <fieldname>]
%%%      or $lsflds <objectname>.<fieldname>
%%%      or $lsflds <dirname>.<objectName> [IND]
%%%     where the optional second parameter is the word "IND" to list only indexed fields
%%%           or is the name (or part of the name) of a field to be listed
%%%
%%% Modified March/2111 Livio -
%%%         Accept syntax <dirname>.<objectname>
%%%         Accept Roles as <objectname>
%%%
if "#<1>" = "?"
   list 14 lsflds 
   return
endif

out "#<1> - #<2> - #<3> "
set save
set output trim on
set headings off
set membercount off
set info off
out
%              Prepare parameters
if "#<1>" = "?"
   list 14 $lsflds 
   return
endif

if "#<1>" <> ""
   let <Filter> = ""
   let <IND>    = ""
   if "#<1>" like "%.%"
      let <OName> = $substring("#<1>",1,$pos("#<1>",'.')-1)
      let <FName> = $substring("#<1>",$pos("#<1>",'.')+1,18)
      let <Filter> = " and fieldname like '#<FName>' "
   else
      let <OName> = $trim("#<1>")
   endif

   if "#<2>" > ""
      if $toupper("#<2>") in("IND","I")
         let <IND> = " and Index <> 'NO'"
      elseif $toupper("#<2>") in ("REQ","REQD","R")
         let <IND> = " and REQD= 'YES'"
      endif
   endif
%              Get the object definition from Objectlist
   let vObjectOwnerName = NULL   vObjectType = Null
   compute Objlist where ObjectName = "#<OName>"  eval ((let vObjectType=ObjectType), \
                                                        (let vObjectOwnerName=OwnerName))
   if vObjectType not in ('Entityset','Relationship','Document','Role')
      out "#<1> is not an Entityset, Relationship, Document or Role. "
      set restore
      return
   elseif vObjectType = "Role"
      let <OName> = $trim(vObjectOwnerName)
      compute objlist where ObjectName = vObjectOwnerName eval (let vdc18 = objecttype)
      let <ObjType> = $substring(vdc18,1,3)
   else
      let <ObjType> = $substring(vObjectType,1,3)
   endif
   if "#<ObjType>s" in ("Ents","Rels")
      compute #<ObjType>s where #<ObjType>Name = "#<OName>"  \
         eval(let vDC18= $concat("Remotename:", RemoteName))
   else
      let vDC18 = "XXXXXXX"
   endif
%
%              List fields
   compute all Flds where ownername = "#<OName>"  #<Filter> #<IND>     \
            sorted by DirName SN FieldName eval ("") -> FldSet
   if $membercount = 0
      compute all Flds where ownername = "#<OName>"?  #<Filter> #<IND>   \
            sorted by dirname Ownername SN FieldName eval ("") -> FldSet
   endif
   if $membercount = 0
      output "No Fields found for Owner: #<1>"
      set restore
      return
   endif

   out 'Fields of "#<1>" ' $squeeze("","(",vObjectType,")")  \
               "#<RemName>"

   li #<ObjType>s where #<ObjType>name = "#<1>"   format '#<ObjType>type:' #<ObjType>Type vDC18

   output " Dirname.OwnerName               SN FieldName          Type    Length Dec Rq Idx Virt RemoteName      Default"
   list all FldSet format \
      $tochar($squeeze('.',dirname,OwnerName),30)   \
      $toalpha(sn,4)  Heading '  SN'\
      fieldname \
      type \
      $toalpha(length,5) heading '  Len' \
      $leftjustify($toalpha(decimals,2)) Heading 'Dec' \
\      $concat("(",$toalpha(listwidth,3),")") heading 'LstW'\
      $toalpha(reqd,3) Heading 'Req'   \
      $toalpha(index,3) Heading 'Ind' \
      $toalpha(virtual,4) Heading 'Virt'\
      $toalpha(remotename,16) heading "RemoteName" \
      $toalpha(default,256) Heading 'Default'
else
   out "Usage: LsFlds <Objectname> [IND]"
   out "   or  LsFlds <ObjectName>.<fieldname> [IND]"
   out "              the object name may be incomplete"
   out "Example: LsFlds cust"
   out '              will list all fields where OwnerName = "CUST"? '
   out
endif
set restore
let <IND> = ""

