﻿%%========================================================================%
%% Break Handler Routine
%%========================================================================%
localprocedure plBreakHandler( vlSaveStatus, inout vlBreakStatus )
   if( vlSaveStatus = $true ) 
      SET RESTORE
   endif
   let vlBreakStatus = $true
endprocedure

%%========================================================================%
%% Adding Fields
%%========================================================================%
localprocedure plFieldAdd( inout vlBreakStatus, \
   vlOwnerNm, vlFldName, vlDataType, vlLength, vlSN, vlSeq, \
   vlIndex, vlReqd, vlVirtual, vlDefault )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <SN> = $trim( vlSN ) \
       <Seq> = $trim( vlSeq ) \
       <OwnerNm> = $trim( vlOwnerNm ) \
       <FldName> = $trim( vlFldName ) \
       <DataType> = $trim( vlDataType ) \
       <Length> = $trim( vlLength ) \
       <Index> = { 'unique' where $tolower(vlIndex) = 'unique', \
                   'yes' where $tolower(vlIndex) = 'yes', \
                   'no' } \
       <Reqd> = { 'yes' where $tolower(vlReqd) = 'yes', 'no' } \
       <Virtual> = { 'yes' where $tolower(vlVirtual) = 'yes', 'no' } \
       <Default> = $trim( vlDefault )
   let <ListWidth> = {'-1' where $tolower('#<Virtual>') = 'yes' or $tolower('#<FldName>') in ('objectkey', 'objecttype', 'nullobjectkey'), '#<Length>' }

   if( $exist$( '#<OwnerNm>', '#<FldName>', 5 ) = $TRUE )
      if( $sysdirid( #<OwnerNm>.#<FldName> ) = 1 ) %% i.e. in ZIM dir
         out '#<OwnerNm>' '#<FldName>' "already exists..."
         RETURN
      endif
   endif

   add 1 Fields \
      let \
         Fields.SN           = '#<SN>' \
         Fields.FieldName    = '#<FldName>' \
         Fields.OwnerName    = '#<OwnerNm>' \
         Fields.Type         = '#<DataType>' \
         Fields.Length       = '#<Length>' \
         Fields.Decimals     = '0' \
         Fields.Reqd         = '#<Reqd>' \
         Fields.Index        = '#<Index>' \
         Fields.Virtual      = '#<Virtual>' \
         Fields.ListWidth    = '#<ListWidth>' \
         Fields.FieldHeading = $null \
         Fields.FieldMask    = $null \
         Fields.Default      = "#<Default>" where "#<Default>" <> '' \
         Fields.EQSel        = '1.000' \
         Fields.BWSel        = '1.000' \
         Fields.LGSel        = '1.000' \
         Fields.DirName      = 'ZIM'
         
   create Field #<OwnerNm> #<FldName> in ZIM $sn #<Seq>

   %% Set permissions
   if '#<Virtual>' <> $toalpha('yes',3)
      permission #<Ownernm>.#<FldName> owner read update
      permission #<Ownernm>.#<FldName> group read update
      permission #<Ownernm>.#<FldName> other read update
   else %% virtual fields are read only of course
      permission #<Ownernm>.#<FldName> owner read
      permission #<Ownernm>.#<FldName> group read
      permission #<Ownernm>.#<FldName> other read
   endif

   delete 1 Fields \
      where \
         Fields.FieldName = '#<FldName>' and \
         Fields.OwnerName = '#<OwnerNm>' and \
         Fields.DirName = 'ZIM'

endprocedure

%%========================================================================%
%% Adding EntitySets
%%========================================================================%
localprocedure plEntAdd( inout vlBreakStatus, \
   vlOwnerNm, vlFldName, vlDataType, vlLength, vlSN, vlSeq, vlFileNum, \
   vlIndex, vlReqd, vlVirtual, vlDefault )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <SN> = $trim( vlSN ) \
       <Seq> = $trim( vlSeq ) \
       <FileNum> = $trim( vlFileNum ) \
       <OwnerNm> = $trim( vlOwnerNm ) \
       <FldName> = $trim( vlFldName ) \
       <DataType> = $trim( vlDataType ) \
       <Length> = $trim( vlLength ) \
       <Index> = { 'unique' where $tolower(vlIndex) = 'unique', \
                   'yes' where $tolower(vlIndex) = 'yes', \
                   'no' } \
       <Reqd> = { 'yes' where $tolower(vlReqd) = 'yes', 'no' } \
       <Virtual> = { 'yes' where $tolower(vlVirtual) = 'yes', 'no' } \
       <Default> = $trim( vlDefault )
   let <ListWidth> = {'-1' where $tolower('#<Virtual>') = 'yes' \
                              or $tolower('#<FldName>') = 'objectkey', \
                      '#<Length>' }

   add 1 EntitySets \
      let \
         EntitySets.EntName = '#<OwnerNm>' \
         EntitySets.EntType = '' \
         EntitySets.AvgSize = '0' \
         EntitySets.DirName = 'ZIM'

   add 1 Fields \
      let \
         Fields.SN           = '#<SN>' \
         Fields.FieldName    = '#<FldName>' \
         Fields.OwnerName    = '#<OwnerNm>' \
         Fields.Type         = '#<DataType>' \
         Fields.Length       = '#<Length>' \
         Fields.Decimals     = '0' \
         Fields.Reqd         = '#<Reqd>' \
         Fields.Index        = '#<Index>' \
         Fields.Virtual      = '#<Virtual>' \
         Fields.ListWidth    = '#<ListWidth>' \
         Fields.FieldHeading = $null \
         Fields.FieldMask    = $null \
         Fields.Default      = "#<Default>" where "#<Default>" <> '' \
         Fields.EQSel        = '1.000' \
         Fields.BWSel        = '1.000' \
         Fields.LGSel        = '1.000' \
         Fields.DirName      = 'ZIM'
         
@timezero 
   create EntitySet #<OwnerNm> in ZIM $fn #<FileNum> $sn #<Seq> delete

   %% Set permissions
   permission ZIM.#<OwnerNm> owner read add change delete
   permission ZIM.#<OwnerNm> group read add change delete
   permission ZIM.#<OwnerNm> other read add change delete

   %% Set field permissions
   if '#<Virtual>' <> $toalpha('yes',3)
      permission #<Ownernm>.#<FldName> owner read update
      permission #<Ownernm>.#<FldName> group read update
      permission #<Ownernm>.#<FldName> other read update
   else %% virtual fields are read only of course
      permission #<Ownernm>.#<FldName> owner read
      permission #<Ownernm>.#<FldName> group read
      permission #<Ownernm>.#<FldName> other read
   endif

   delete 1 Ents \
      where Ents.EntName = '#<OwnerNm>' and Ents.DirName = 'ZIM'

   delete 1 Fields \
      where Fields.FieldName = '#<FldName>' \
        and Fields.OwnerName = '#<OwnerNm>' \
        and Fields.DirName = 'ZIM'

endprocedure

%%========================================================================%
%% Adding Constants
%%========================================================================%
localprocedure plConstAdd( inout vlBreakStatus, vlConstName, vlDataType, vlValue, vlSeq)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <ConstName> = $trim( vlConstName ) \
       <DataType> = $trim( vlDataType ) \
       <ConstValue> = $trim( vlValue ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<ConstName>', 18 ) = $FALSE
      add 1 Consts \
         let \
            Consts.ConstName = '#<ConstName>' \
            Consts.DirName = 'ZIM' \
            Consts.ConstType = '#<DataType>' \
            Consts.ConstValue = '#<ConstValue>'
            
      create constant #<ConstName> in ZIM $sn #<Seq>

      delete 1 Consts \
         where \
             Consts.ConstName = '#<ConstName>' and \
             Consts.DirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%% Updating Constants
%%========================================================================%
localprocedure plConstUpdate( inout vlBreakStatus, \
                                    vlConstName, \
                                    vlDataType, \
                                    vlValue, \
                                    vlSeq)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <ConstName> = $trim( vlConstName ) \
       <DataType> = $trim( vlDataType ) \
       <ConstValue> = $trim( vlValue ) \
       <Seq> = $trim( vlSeq )

   delete 1 Consts \
      where Consts.ConstName = '#<ConstName>' and Consts.DirName = 'ZIM'

   if $exist$('ZIM', '#<ConstName>', 18) = $true
@syser
      erase constant ZIM.#<ConstName> from ZIM
   endif

   add 1 Consts \
      let \
         Consts.ConstName = '#<ConstName>' \
         Consts.DirName = 'ZIM' \
         Consts.ConstType = '#<DataType>' \
         Consts.ConstValue = '#<ConstValue>'
         
   create constant #<ConstName> in ZIM $sn #<Seq>

   delete 1 Consts \
      where Consts.ConstName = '#<ConstName>' and Consts.DirName = 'ZIM'
endprocedure

%%========================================================================%
%% Adding Documents
%%========================================================================%
localprocedure plDocAdd( inout vlBreakStatus, vlDocName, \
      vlFileName, vlDocType, vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <DocName> = $trim( vlDocName ) \
       <FileName> = $trim( vlFileName ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<DocName>', 9 ) = $FALSE
      add 1 Docs \
         let \
            Docs.DocName = '#<DocName>' \
            Docs.FileName = '#<FileName>' \
            Docs.DocType = $trim(vlDocType) \
            Docs.DirName = 'ZIM'
            
      create document #<DocName> in ZIM $sn #<Seq>
       delete 1 Docs \
          where \
             Docs.DocName = '#<DocName>' and \
             Docs.DirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%% Adding Roles
%%========================================================================%
localprocedure plRoleAdd( inout vlBreakStatus, \
                                vlRoleName, \
                                vlRoleOwner, \
                                vlRoleCondition, \
                                vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <RoleName> = $trim( vlRoleName ) \
       <RoleOwner> = $trim( vlRoleOwner ) \
       <RoleCondition> = $trim( vlRoleCondition ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<RoleName>', 12 ) = $FALSE
      add 1 Roles \
         let \
            Roles.RoleName = '#<RoleName>' \
            Roles.OwnerName = '#<RoleOwner>' \
            Roles.RoleCondition = '#<RoleCondition>' \
            Roles.DirName = 'ZIM'
            
      create role #<RoleName> in ZIM $sn #<Seq>
      delete roles where \
            Roles.RoleName = '#<RoleName>' and \
            Roles.DirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%% Adding Relationships
%%========================================================================%
localprocedure plRelAdd( inout vlBreakStatus, vlRelName, vlRelExpr, vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <RelName> = $trim( vlRelName ) \
       <RelExpr> = $trim( vlRelExpr ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<RelName>', 6 ) = $FALSE        %% NODATAREL !!
      add 1 Rels \
         let \
            Rels.RelName = '#<RelName>' \
            Rels.RelCondition = '#<RelExpr>' \
            Rels.DirName = 'ZIM'
            
      create relationship #<RelName> in ZIM $sn #<Seq> delete

      delete 1 Rels \
         where \
            Rels.RelName = '#<RelName>' and \
            Rels.DirName = 'ZIM'
   endif
endprocedure

%%========================================================================%
%% Adding Directories
%%========================================================================%
localprocedure plDirAdd( inout vlBreakStatus, vlDirName, vlDirID, vlSeq )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   let <DirName> = $trim( vlDirName ) \
       <DirID> = $trim( vlDirID ) \
       <Seq> = $trim( vlSeq )

   if $exist$( 'ZIM', '#<DirName>', 2 ) = $FALSE
      add 1 Directories \
         let \
            Directories.DirName = '#<DirName>' \
            Directories.OwnerDirName = 'ZIM' \
            Directories.DirId = '#<DirID>' \
            Directories.Foreign = 'yes'
            
      create Directory #<DirName> in ZIM $sn #<Seq> delete

      %% Set permissions
      permission ZIM.#<DirName> owner read update
      permission ZIM.#<DirName> group read update
      permission ZIM.#<DirName> other read update

      delete 1 Dirs \
         where \
            Dirs.DirName = '#<DirName>' and \
            Dirs.OwnerDirName = 'ZIM'
   endif
endprocedure

%
%       Program to add special Event, Window, Form, and Context
%               structures. These are created as incore entity sets.
%               They must be created in the ZIM dir with system names
%               less than 1000 and will be sorted by SN which should
%               be set to the desired system name.
%
%       Currently assigned system name ranges are:
%
%       Event           - 900, fields 901 to 915
%       ThisWindow      - 920, fields 921 to 938
%       ThisForm        - 940, fields 941 to 958
%       ThisMenu        - 970, fields 971 to 975

localprocedure  plAddStructureFld( pEntName, pFieldName, pDataType, \
                                        pLength, pSN )

add fields let \
        SN = pSN \
        FieldName = pFieldName \
        OwnerName = pEntName \
        Type = pDataType \
        Length = pLength \
        Decimals = 0 \
        DirName = 'ZIM'

endprocedure

%%========================================================================%
% plAddContext: Adding Window Context Structures (if necessary)
%%========================================================================%
localprocedure plAddContext ( vl45To50, inout vlBreakStatus ) \
                   local (vlSaveStatus)

on BREAK
   plBreakHandler( vlSaveStatus, vlBreakStatus )
   RETURN
endon

   SET SAVE
   let vlSaveStatus = $true

   set warnings off

%%   output 'Adding Window Context Structures (if necessary)'

   %% EVENT structure

   if $Exist$ ('ZIM', 'Event', 20) <> $true
      delete all ents where entname = 'Event' and dirname = 'ZIM'
      delete all fields wh ownername = 'Event' and dirname = 'ZIM'

      add ents let entname = 'Event' dirname = 'ZIM'

      plAddStructureFld( 'Event', 'EventName',      'Alpha', 20, 901 )
      plAddStructureFld( 'Event', 'EventType',      'Alpha', 20, 902 )
      plAddStructureFld( 'Event', 'EventTag',       'Alpha', 18, 903 )
      plAddStructureFld( 'Event', 'WindowTag',      'Alpha', 18, 904 )
      plAddStructureFld( 'Event', 'FormTag',        'Alpha', 18, 905 )
      plAddStructureFld( 'Event', 'FieldTag',       'Alpha', 18, 906 )
      plAddStructureFld( 'Event', 'MenuTag',        'Alpha', 18, 907 )
      plAddStructureFld( 'Event', 'MenuItemTag',    'Alpha', 18, 908 )
      plAddStructureFld( 'Event', 'AcceleratorKey', 'Alpha', 18, 909 )
      plAddStructureFld( 'Event', 'ScrollingKey',   'Char',  1,  910 )
      plAddStructureFld( 'Event', 'KeyPressed',     'Char',  1,  911 )
      plAddStructureFld( 'Event', 'MouseClick',     'Char',  1,  912 )
      plAddStructureFld( 'Event', 'ShiftKey',       'Char',  1,  913 )
      plAddStructureFld( 'Event', 'CtrlKey',        'Char',  1,  914 )
      plAddStructureFld( 'Event', 'AltKey',         'Char',  1,  915 )

      create form entity Event in ZIM $SN 900

      delete all ents where entname = 'Event' and dirname = 'ZIM'
      delete all fields wh ownername = 'Event' and dirname = 'ZIM'
   endif

   % THISWINDOW structure

   if $Exist$ ('ZIM', 'ThisWindow', 20) <> $true
      delete all ents where entname = 'ThisWindow' and dirname = 'ZIM'
      delete all fields wh OwnerName = 'ThisWindow' and dirname = 'ZIM'

      add ents let entname = 'ThisWindow' dirname = 'ZIM'

      plAddStructureFld( 'ThisWindow', 'WindowName',    'Alpha', 18, 921 )
      plAddStructureFld( 'ThisWindow', 'WindowTag',     'Alpha', 18, 922 )
      plAddStructureFld( 'ThisWindow', 'WindowNum',     'Int',   2,  923 )
      plAddStructureFld( 'ThisWindow', 'WindowState',   'Int',   2,  924 )
      plAddStructureFld( 'ThisWindow', 'EventName',     'Alpha', 20, 925 )
      plAddStructureFld( 'ThisWindow', 'EventType',     'Alpha', 20, 926 )
      plAddStructureFld( 'ThisWindow', 'EventTag',      'Alpha', 18, 927 )
      plAddStructureFld( 'ThisWindow', 'AcceleratorKey','Alpha', 18, 928 )
      plAddStructureFld( 'ThisWindow', 'ScrollingKey',  'Char',  1,  929 )
      plAddStructureFld( 'ThisWindow', 'KeyPressed',    'Char',  1,  930 )
      plAddStructureFld( 'ThisWindow', 'MouseClick',    'Char',  1,  931 )
      plAddStructureFld( 'ThisWindow', 'ShiftKey',      'Char',  1,  932 )
      plAddStructureFld( 'ThisWindow', 'CtrlKey',       'Char',  1,  933 )
      plAddStructureFld( 'ThisWindow', 'AltKey',        'Char',  1,  934 )
      plAddStructureFld( 'ThisWindow', 'WindowCol',     'Int',   2,  935 )
      plAddStructureFld( 'ThisWindow', 'WindowRow',     'Int',   2,  936 )
      plAddStructureFld( 'ThisWindow', 'WindowHeight',  'Int',   2,  937 )
      plAddStructureFld( 'ThisWindow', 'WindowWidth',   'Int',   2,  938 )

      create form entity ThisWindow in ZIM $SN 920

      delete all ents where entname = 'ThisWindow' and dirname = 'ZIM'
      delete all fields wh ownername = 'ThisWindow' and dirname = 'ZIM'
   endif

   % THISFORM structure

   if $Exist$ ('ZIM', 'ThisForm', 20) <> $true
      delete all ents where entname = 'ThisForm' and dirname = 'ZIM'
      delete all fields wh OwnerName = 'ThisForm' and dirname = 'ZIM'

      add ents let entname = 'ThisForm' dirname = 'ZIM'

      plAddStructureFld( 'ThisForm', 'DisplayTag',  'Alpha', 18, 941 )
      plAddStructureFld( 'ThisForm', 'FormTag',     'Alpha', 18, 942 )
      plAddStructureFld( 'ThisForm', 'FormNum',     'Int',   2,  943 )
      plAddStructureFld( 'ThisForm', 'FormChanged', 'Char',  1,  944 )
      plAddStructureFld( 'ThisForm', 'FieldTag',    'Alpha', 18, 945 )
      plAddStructureFld( 'ThisForm', 'FFFieldNum',  'Int',   2,  946 )
      plAddStructureFld( 'ThisForm', 'Subscript',   'Int',   2,  947 )
      plAddStructureFld( 'ThisForm', 'FieldChanged','Char',  1,  948 )
      plAddStructureFld( 'ThisForm', 'ReqError',    'Char',  1,  949 )
      plAddStructureFld( 'ThisForm', 'ReqTag',      'Alpha', 18, 950 )
      plAddStructureFld( 'ThisForm', 'ReqFormNum',  'Int',   2,  951 )
      plAddStructureFld( 'ThisForm', 'ReqFieldNum', 'Int',   2,  952 )
      plAddStructureFld( 'ThisForm', 'ReqSubscript','Int',   2,  953 )
      plAddStructureFld( 'ThisForm', 'ValError',    'Char',  1,  954 )
      plAddStructureFld( 'ThisForm', 'ValTag',      'Alpha', 18, 955 )
      plAddStructureFld( 'ThisForm', 'ValFormNum',  'Int',   2,  956 )
      plAddStructureFld( 'ThisForm', 'ValFieldNum', 'Int',   2,  957 )
      plAddStructureFld( 'ThisForm', 'ValSubscript','Int',   2,  958 )
      
      create form entity ThisForm in ZIM $SN 940

      delete all ents where entname = 'ThisForm' and dirname = 'ZIM'
      delete all fields wh ownername = 'ThisForm' and dirname = 'ZIM'
   endif

   % THISMENU structure

   if $Exist$ ('ZIM', 'ThisMenu', 20) <> $true
      delete all ents where entname = 'ThisMenu' and dirname = 'ZIM'
      delete all fields wh OwnerName = 'ThisMenu' and dirname = 'ZIM'

      add ents let entname = 'ThisMenu' dirname = 'ZIM'
      plAddStructureFld( 'ThisMenu', 'MenuTag',     'Alpha', 18, 971 )
      plAddStructureFld( 'ThisMenu', 'MenuNum',     'Int',   2,  972 )
      plAddStructureFld( 'ThisMenu', 'MenuChanged', 'Char',  1,  973 )
      plAddStructureFld( 'ThisMenu', 'MenuItemTag', 'Alpha', 18, 974 )
      plAddStructureFld( 'ThisMenu', 'MenuItemNum', 'Int',   2,  975 )
      
      create form entity ThisMenu in ZIM $SN 970

      delete all ents where entname = 'ThisMenu' and dirname = 'ZIM'
      delete all fields wh ownername = 'ThisMenu' and dirname = 'ZIM'
   endif

   SET RESTORE

endprocedure

%%========================================================================%
% Adding in System Constants for ZIM 4.1 (if necessary)
%%========================================================================%
localprocedure plZIM41Add( inout vlBreakStatus )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% Note - $sn 549 - 599 used by 4.0 to 4.1 extension procedure
   %% ZIMINIT table has reserved from $sn 540 - 599 for ZOM extensions
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   % NullDirName is useless but cannot be removed unless NullFormDirName, ObjectKey and NullObjectKey are also removed.
   plFieldAdd( vlBreakStatus, \
      'DisplayForms', 'NullDirName', 'alpha', '1', '4103', '578', \
      'yes', 'no', 'yes', "'Y' where DirName is $null or DirName = ''" )
   % NullDirName is useless but cannot be removed unless NullFormDirName, ObjectKey and NullObjectKey are also removed.

   plFieldAdd( vlBreakStatus, \
      'DisplayForms', 'NullFormDirName', 'alpha', '1', '4104', '579', \
      'yes', 'no', 'yes', "'Y' where FormDirName is $null or FormDirName = ''" )
   plFieldAdd( vlBreakStatus, \
      'Displays', 'NullObjectKey', 'alpha', '1', '4102', '575', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   plFieldAdd( vlBreakStatus, \
      'Displays', 'ObjectType', 'alpha', '12', '4105', '577', \
      'no', 'no', 'yes', "'Display'" )
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Documents', 'NullObjectKey', 'alpha', '1', '4102', '563', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Documents', 'ObjectType', 'alpha', '12', '4105', '565', \
      'no', 'no', 'yes', "'Document'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'EntitySets', 'NullObjectKey', 'alpha', '1', '4102', '551', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'EntitySets', 'ObjectType', 'alpha', '12', '4105', '553', \
      'no', 'no', 'yes', "'EntitySet'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Relationships', 'NullObjectKey', 'alpha', '1', '4102', '555', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Relationships', 'ObjectType', 'alpha', '12', '4105', '557', \
      'no', 'no', 'yes', "'Relationship'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Roles', 'NullObjectKey', 'alpha', '1', '4102', '559', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Roles', 'ObjectType', 'alpha', '12', '4105', '561', \
      'no', 'no', 'yes', "'Role'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Constants', 'NullObjectKey', 'alpha', '1', '4102', '594', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Constants', 'ObjectType', 'alpha', '12', '4105', '596', \
      'no', 'no', 'yes', "'Constant'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Directories', 'NullObjectKey', 'alpha', '1', '4102', '598', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %% SN 549 - ran out and went back
   plFieldAdd( vlBreakStatus, \
      'Directories', 'ObjectType', 'alpha', '12', '4105', '549', \
      'no', 'no', 'yes', "'Directory'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Variables', 'NullObjectKey', 'alpha', '1', '4102', '586', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Variables', 'ObjectType', 'alpha', '12', '4105', '588', \
      'no', 'no', 'yes', "'Variable'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Windows', 'NullObjectKey', 'alpha', '1', '4102', '571', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Windows', 'ObjectType', 'alpha', '12', '4105', '573', \
      'no', 'no', 'yes', "'Window'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Fields', 'NullObjectKey', 'alpha', '1', '4102', '567', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Fields', 'ObjectType', 'alpha', '12', '4105', '569', \
      'no', 'no', 'yes', "'Field'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Forms', 'NullObjectKey', 'alpha', '1', '4102', '581', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'Forms', 'ObjectType', 'alpha', '12', '4105', '583', \
      'no', 'no', 'yes', "'Form'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'NamedSets', 'NullObjectKey', 'alpha', '1', '4102', '590', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'NamedSets', 'ObjectType', 'alpha', '12', '4105', '592', \
      'no', 'no', 'yes', "'Set'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   output 'Adding Services Directories for 4.1 (if necessary)'
   plDirAdd( vlBreakStatus, '$DeployServices', '250', '999' )
   if( vlBreakStatus = $true )
      set exception break
   endif

endprocedure

%%========================================================================%
% Adding in System Constants for ZIM 4.5 (if necessary)
%%========================================================================%
localprocedure plZIM45Add( inout vlBreakStatus )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

%%   output 'Adding in System Constants for ZIM 5.0'

   plConstAdd( vlBreakStatus, 'Modified', 'numeric', '62', '422' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Click', 'numeric', '63', '423' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'DoubleClick', 'numeric', '64', '424' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'TabIndex', 'numeric', '90', '446' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cActiveBorder', 'numeric', '37', '425' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cActiveCaption', 'numeric', '36', '426' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cAppWorkSpace', 'numeric', '35', '427' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cWindowBackground', 'numeric', '34', '428' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cButtonFace', 'numeric', '33', '429' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cButtonShadow', 'numeric', '32', '430' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cButtonText', 'numeric', '31', '431' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cCaptionText', 'numeric', '30', '432' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cGreyText', 'numeric', '29', '467' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cGrayText', 'numeric', '29', '433' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cHighlightColor', 'numeric', '28', '434' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cHighlightText', 'numeric', '27', '435' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cInactiveBorder', 'numeric', '26', '436' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cInactiveCaption', 'numeric', '25', '437' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cMenuColor', 'numeric', '24', '438' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cMenuTextColor', 'numeric', '23', '439' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cScrollBarColor', 'numeric', '22', '440' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cWindowFrame', 'numeric', '21', '441' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cWindowText', 'numeric', '20', '442' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cInactivCaptionTxt', 'numeric', '19', '443' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cButtonHighlight', 'numeric', '18', '444' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'cWindowColor', 'numeric', '17', '445' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'GotFocus', 'numeric', '92', '447' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'LostFocusModified', 'numeric', '36', '448' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'LostFocus', 'numeric', '59', '449' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Closed', 'numeric', '65', '450' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Checked', 'numeric', '95', '458' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Italic', 'numeric', '45', '460' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Strikeout', 'numeric', '46', '461' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Bold', 'numeric', '47', '462' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Unavailable', 'numeric', '48', '463' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'Available', 'numeric', '37', '464' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'LightGray', 'numeric', '8', '465' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'DarkGray', 'numeric', '9', '466' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %% Note - $sn 545 - 548 used by 4.1 to 5.0 extension procedure
   %% for ZOM. ZIMINIT table has reserved from $sn 540 - 599 for
   %% ZOM extensions.
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%   output 'Upgrading Dictionary to 5.0'

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'DisplayForms', 'ObjectKey', 'alpha', '25', '4511', '801', \
      'unique', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DisplayForms', 'NullObjectKey', 'alpha', '1', '4512', '802', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DisplayForms', 'ObjectType', 'alpha', '12', '4513', '803', \
      'no', 'no', 'yes', "'DisplayForm'" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   set warnings off

   % NullDirName is useless but cannot be removed unless ObjectKey and NullObjectKey are also removed.
   plFieldAdd( vlBreakStatus, \
      'FormFields', 'NullDirName', 'alpha', '1', '4103', '584', \
      'yes', 'no', 'yes', "'Y' where DirName is $null or DirName = ''" )
   % NullDirName is useless but cannot be removed unless ObjectKey and NullObjectKey are also removed.

   plFieldAdd( vlBreakStatus, \
      'FormFields', 'ObjectKey', 'alpha', '25', '4557', '544', \
      'unique', 'no', 'no', '' )
   plFieldAdd( vlBreakStatus, \
      'FormFields', 'NullObjectKey', 'alpha', '1', '4559', '543', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   plFieldAdd( vlBreakStatus, \
      'FormFields', 'ObjectType', 'alpha', '12', '4560', '542', \
      'no', 'no', 'yes', "'FormField'" )
   set warnings on

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plFieldAdd( vlBreakStatus, \
      'Menus', 'NullObjectKey', 'alpha', '1', '4526', '547', \
      'yes', 'no', 'yes', "'Y' where ObjectKey is $null" )
   plFieldAdd( vlBreakStatus, \
      'Menus', 'ObjectType', 'alpha', '12', '4528', '545', \
      'no', 'no', 'yes', "'Menu'" )

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   plDocAdd( vlBreakStatus, \
      'ClipBoard', '$clipboard', 'device', '520' )
   if( vlBreakStatus = $true )
      set exception break
   endif

endprocedure

%%========================================================================%
% Upgrading from 4.5 to 5.0 (if necessary)
%%========================================================================%
localprocedure plZIM50Add( vl45To50, inout vlBreakStatus ) \
                   local (vlSaveStatus)
on BREAK
   plBreakHandler( vlSaveStatus, vlBreakStatus )
   RETURN
endon

   SET SAVE
   let vlSaveStatus = $true

   if vl45To50 = $true
%%      output 'Upgrading from 4.5 to 5.0 (if necessary)'
   endif
%% FR/Livio 12/05/2004
%%                                             plObjStatus('ZIM.Windows.WdgStyle')
%%                                             if $LastErrCode <> 0
   if $exist$('Windows','WdgStyle',5) = 0
%%      output 'Adding Windows.WdgStyle'
      plFieldAdd( vlBreakStatus, \
         'Windows', 'WdgStyle', 'Int', '2', \
         '4550', '699', 'no', 'no', 'no', '' )
      if( vlBreakStatus = $true )
         set exception break
      endif

      if vl45To50 = $true
         %% update windows WdgStyle here
         change all Wins let \
            WdgStyle = { \
               0 where Wins.Bord = 'no', \
               3 where WdgResize = 1, \
               1 where WdgBorderStyle = 1, \
               2}
      endif
   endif

   if $exist$('FormFields','WdgItems',5) = 0
      % Test whether the database is Unicode.
      add 1 Consts let constname="$TestUnicode$" constvalue="测试数据库是否为Unicode" consttype="Unichar" dirname="ZIM" -> sTestUnicode
      find 1 Consts where constvalue="????????Unicode"
      if $setcount = 1
         plFieldAdd( vlBreakStatus, 'FormFields', 'WdgItems', 'varalpha', '256', '4558', '656', 'no', 'no', 'no', '' )
      else
         plFieldAdd( vlBreakStatus, 'FormFields', 'WdgItems', 'Univar', '256', '4558', '656', 'no', 'no', 'no', '' )
      endif
      plFieldAdd( vlBreakStatus, 'FormFields', 'WdgType', 'int', '3', '4600', '495', 'no', 'no', 'no', "" )
      find 1 Consts where constvalue="????????Unicode"
      if $setcount = 1
         plFieldAdd( vlBreakStatus, 'FormFields', 'WdgCSS', 'varchar', '512', '4610', '496', 'no', 'no', 'no', "" )
      else
         plFieldAdd( vlBreakStatus, 'FormFields', 'WdgCSS', 'Univar', '512', '4610', '496', 'no', 'no', 'no', "" )
      endif
      delete sTestUnicode
  	
      if vl45To50 = $true
         %% Adjust all lists and combo boxes of 'old' format
         %% Use WdgId because it is indexed and this will avoid TUI forms
         change all FFs where WdgId > 0 and FT in ('L', 'C') let \
            WdgItems = $translate(ErrorMsg, ',', ';') \
            ErrorMsg = ''

%%         output 'Checking that all Frames and Option Button Boxes are Available'
         %% make all frames and option button boxes unprotected.
         %% Use WdgId because it is indexed and this will avoid TUI forms
         change all FFs where WdgId > 0 and FT in ('F', 'O') and (PM <> 'U' or WdgTabStop <> 1) \
            let PM = 'U' WdgTabStop = 1
      endif
   endif
   
   if $exist$('DDDescriptions','ObjectKey',5) = 0
%%      output 'Adding DDDescriptions.ObjectKey'
      plFieldAdd( vlBreakStatus, 'DDDescriptions', 'ObjectKey', 'alpha', '25', '4000', '804', 'unique', 'no', 'no', '' )
   endif

   plAddContext (vl45To50, vlBreakStatus)

   if $Exist$('ZIM','MenuItems',12) = 0
      plRoleAdd( vlBreakStatus, 'MenuItems', 'FormFields', '', 519 )
      if( vlBreakStatus = $true )
         set exception break
      endif
   endif

   SET RESTORE
   let vlSaveStatus = $false
endprocedure

localprocedure plODv52r10Update( inout vlBreakStatus ) local (vlGet, vlSet)
   %% Do the upgrade from ZIM 4.0 to ZIM 4.1
   plZIM41Add( vlBreakStatus )

   %% Do the upgrade from ZIM 4.1 to ZIM 5.0
   plZIM45Add( vlBreakStatus )
   plZIM50Add( $false, vlBreakStatus )

   %% Upgrade from ZIM 5.0 to OD v5.2 r1.0
   plConstAdd( vlBreakStatus, 'TableType', 'numeric', '42', '489' )
   plConstAdd( vlBreakStatus, 'RemOwner', 'numeric', '43', '490' )
   plConstAdd( vlBreakStatus, 'RemName', 'numeric', '49', '491' )
   plConstAdd( vlBreakStatus, 'ReadOnlyLocks', 'numeric', '116', '493' )
   plConstAdd( vlBreakStatus, 'Locks', 'numeric', '117', '494' )

   set ZIM.FormFields.BC required off
   set ZIM.FormFields.FC required off
   set ZIM.Menus.BC required off
   set ZIM.Menus.FC required off
   set ZIM.Windows.BC required off
   set ZIM.Windows.FC required off

   set ZIM.DisplayForms.Row required off
   set ZIM.DisplayForms.Col required off
   set ZIM.FormFields.Row required off
   set ZIM.FormFields.Col required off
   set ZIM.Menus.Row required off
   set ZIM.Menus.Col required off
   set ZIM.Windows.Row required off
   set ZIM.Windows.Col required off

   set ZIM.Displays.Hght required off
   set ZIM.Displays.Wdth required off
   set ZIM.Forms.Hght required off
   set ZIM.Forms.Wdth required off
   set ZIM.FormFields.Hght required off
   set ZIM.FormFields.Wdth required off
   set ZIM.Windows.Hght required off
   set ZIM.Windows.Wdth required off

   %% The following updates aren't specifically part of 5.2, but are
   %% here so that they won't need to do them again after this!

   %% Special check for constant 'label' that existed in Beta but is now a
   %% keyword so we need to blow it away!
   if $Exist$ ('ZIM', 'label', 18) = $True
@syser
      erase constant ZIM.label from ZIM
   endif
   set errors on

endprocedure


%%========================================================================%
% Upgrading Zim
%%========================================================================%
localprocedure plODv53r10Update( inout vlBreakStatus )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   plRoleAdd( vlBreakStatus, 'ODDescriptions', 'DDDescriptions', '', 516 )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plRoleAdd( vlBreakStatus, 'ODDescs', 'DDDescriptions', '', 517 )
   if( vlBreakStatus = $true )
      set exception break
   endif

endprocedure

%%========================================================================%
% Setting some fields: remowner (Remote Owner) and remname (Remote Name)
%%========================================================================%
localprocedure plSetRemote()

   set EntitySets remowner ''
   set Fields remowner ''
   set Relationships remowner ''
   set Roles remowner ''
   set Documents remowner ''
   set Forms remowner ''
   set FormFields remowner ''
   set Variables remowner ''
   set Displays remowner ''
   set DisplayForms remowner ''
   set DDDescriptions remowner ''
   set Users remowner ''
   set Constants remowner ''
   set NamedSets remowner ''
   set Windows remowner ''
   set Menus remowner ''
   set PainterGroups remowner ''
   set DefinitionsLog remowner ''

   set EntitySets remname ''
   set Fields remname ''
   set Relationships remname ''
   set Roles remname ''
   set Documents remname ''
   set Forms remname ''
   set FormFields remname ''
   set Variables remname ''
   set Displays remname ''
   set DisplayForms remname ''
   set DDDescriptions remname ''
   set Users remname ''
   set Constants remname ''
   set NamedSets remname ''
   set Windows remname ''
   set Menus remname ''
   set PainterGroups remname ''
   set DefinitionsLog remname ''

   set EntitySets.EntName remname ''
   set EntitySets.EntType remname ''
   set EntitySets.AvgSize remname ''
   set EntitySets.DirName remname ''
   set EntitySets.UniqueEntDirName remname ''
   set EntitySets.ObjectKey remname ''
   set EntitySets.RemoteName remname ''
   set EntitySets.RemoteOwnerName remname ''
   set EntitySets.NullObjectKey remname ''
   set EntitySets.ObjectType remname ''

   set Fields.SN remname ''
   set Fields.FieldName remname ''
   set Fields.OwnerName remname ''
   set Fields.Type remname ''
   set Fields.Length remname ''
   set Fields.Decimals remname ''
   set Fields.Reqd remname ''
   set Fields.Index remname ''
   set Fields.Virtual remname ''
   set Fields.ListWidth remname ''
   set Fields.FieldHeading remname ''
   set Fields.FieldMask remname ''
   set Fields.Default remname ''
   set Fields.EQSel remname ''
   set Fields.BWSel remname ''
   set Fields.LGSel remname ''
   set Fields.DirName remname ''
   set Fields.OwnerDirName remname ''
   set Fields.ObjectKey remname ''
   set Fields.RemoteName remname ''
   set Fields.NullObjectKey remname ''
   set Fields.ObjectType remname ''

   set Relationships.RelName remname ''
   set Relationships.RelCondition remname ''
   set Relationships.RelType remname ''
   set Relationships.AvgSize remname ''
   set Relationships.DirName remname ''
   set Relationships.UniqueRelDirName remname ''
   set Relationships.ObjectKey remname ''
   set Relationships.RemoteName remname ''
   set Relationships.RemoteOwnerName remname ''
   set Relationships.NullObjectKey remname ''
   set Relationships.ObjectType remname ''

   set Roles.RoleName remname ''
   set Roles.OwnerName remname ''
   set Roles.RoleCondition remname ''
   set Roles.DirName remname ''
   set Roles.UniqueRoleDirName remname ''
   set Roles.OwnerDirName remname ''
   set Roles.ObjectKey remname ''
   set Roles.NullObjectKey remname ''
   set Roles.ObjectType remname ''

   set Forms.FormName remname ''
   set Forms.DirName remname ''
   set Forms.UniqueFormDirName remname ''
   set Forms.ObjectKey remname ''
   set Forms.NullObjectKey remname ''
   set Forms.ObjectType remname ''
   set Forms.Wdth remname ''
   set Forms.Hght remname ''
   set Forms.WdgWdth remname ''
   set Forms.WdgHght remname ''
   set Forms.HMargin remname ''
   set Forms.VMargin remname ''
   set Forms.WdgHMargin remname ''
   set Forms.WdgVMargin remname ''
   set Forms.WdgTag remname ''

   set FormFields.FormName remname ''
   set FormFields.FieldName remname ''
   set FormFields.FNum remname ''
   set FormFields.FFSN remname ''
   set FormFields.Row remname ''
   set FormFields.Col remname ''
   set FormFields.Wdth remname ''
   set FormFields.Hght remname ''
   set FormFields.FieldLen remname ''
   set FormFields.FT remname ''
   set FormFields.DT remname ''
   set FormFields.PM remname ''
   set FormFields.DM remname ''
   set FormFields.BC remname ''
   set FormFields.FC remname ''
   set FormFields.IT remname ''
   set FormFields.TE remname ''
   set FormFields.VR remname ''
   set FormFields.SK remname ''
   set FormFields.AC remname ''
   set FormFields.Case remname ''
   set FormFields.Just remname ''
   set FormFields.Fill remname ''
   set FormFields.MinLen remname ''
   set FormFields.AN remname ''
   set FormFields.CM remname ''
   set FormFields.VA remname ''
   set FormFields.DataMask remname ''
   set FormFields.ValRule remname ''
   set FormFields.Default remname ''
   set FormFields.HelpMsg remname ''
   set FormFields.ErrorMsg remname ''
   set FormFields.DirName remname ''
   set FormFields.FFSDirName remname ''
   set FormFields.WdgTag remname ''
   set FormFields.WdgId remname ''
   set FormFields.WdgParId remname ''
   set FormFields.WdgSubClass remname ''
   set FormFields.WdgStyle remname ''
   set FormFields.WdgRow remname ''
   set FormFields.WdgCol remname ''
   set FormFields.WdgWdth remname ''
   set FormFields.WdgHght remname ''
   set FormFields.WdgOrientation remname ''
   set FormFields.WdgFCRed remname ''
   set FormFields.WdgFCGreen remname ''
   set FormFields.WdgFCBlue remname ''
   set FormFields.WdgBCRed remname ''
   set FormFields.WdgBCGreen remname ''
   set FormFields.WdgBCBlue remname ''
   set FormFields.WdgBackground remname ''
   set FormFields.WdgBorderStyle remname ''
   set FormFields.WdgShadowSize remname ''
   set FormFields.WdgLabel remname ''
   set FormFields.WdgOnImage remname ''
   set FormFields.WdgOffImage remname ''
   set FormFields.WdgNullImage remname ''
   set FormFields.WdgFont remname ''
   set FormFields.WdgFontSize remname ''
   set FormFields.WdgPointerStyle remname ''
   set FormFields.WdgMultiLine remname ''
   set FormFields.WdgAutoSize remname ''
   set FormFields.WdgDragIcon remname ''
   set FormFields.WdgOnValue remname ''
   set FormFields.WdgOffValue remname ''
   set FormFields.WdgNullValue remname ''
   set FormFields.WdgMinValue remname ''
   set FormFields.WdgMaxValue remname ''
   set FormFields.WdgLargeIncrement remname ''
   set FormFields.WdgSmallIncrement remname ''
   set FormFields.WdgZOrder remname ''
   set FormFields.WdgTabStop remname ''
   set FormFields.WdgAutoSelect remname ''
   set FormFields.WdgScrollBars remname ''
   set FormFields.WdgEditOptions remname ''
   set FormFields.WdgNumVisibleLines remname ''
   set FormFields.WdgIsDefault remname ''
   set FormFields.WdgDragMode remname ''
   set FormFields.WdgDropTarget remname ''
   set FormFields.WdgModified remname ''
   set FormFields.WdgClick remname ''
   set FormFields.WdgDblClick remname ''
   set FormFields.WdgAutoDrop remname ''
   set FormFields.WdgLabelPosition remname ''
   set FormFields.WdgNumVisibleCols remname ''
   set FormFields.HMargin remname ''
   set FormFields.VMargin remname ''
   set FormFields.WdgHMargin remname ''
   set FormFields.WdgVMargin remname ''
   set FormFields.WdgInputMask remname ''
   set FormFields.ObjectKey remname ''
   set FormFields.NullObjectKey remname ''
   set FormFields.ObjectType remname ''
   set FormFields.WdgItems remname ''

   set Variables.VarName remname ''
   set Variables.Type remname ''
   set Variables.Length remname ''
   set Variables.Decimals remname ''
   set Variables.Size remname ''
   set Variables.DirName remname ''
   set Variables.UniqueVarDirName remname ''
   set Variables.ObjectKey remname ''
   set Variables.NullObjectKey remname ''
   set Variables.ObjectType remname ''

   set Displays.DisplayName remname ''
   set Displays.DirName remname ''
   set Displays.UniqueDispDirName remname ''
   set Displays.ObjectKey remname ''
   set Displays.NullObjectKey remname ''
   set Displays.ObjectType remname ''
   set Displays.WdgTag remname ''
   set Displays.HMargin remname ''
   set Displays.VMargin remname ''
   set Displays.WdgHMargin remname ''
   set Displays.WdgVMargin remname ''
   set Displays.WdgWdth remname ''
   set Displays.WdgHght remname ''
   set Displays.Wdth remname ''
   set Displays.Hght remname ''
   set DisplayForms.DisplayName remname ''
   set DisplayForms.FormName remname ''
   set DisplayForms.FormNum remname ''
   set DisplayForms.FSN remname ''
   set DisplayForms.Row remname ''
   set DisplayForms.Col remname ''
   set DisplayForms.LoadOrder remname ''
   set DisplayForms.RD remname ''
   set DisplayForms.RA remname ''
   set DisplayForms.FormDirName remname ''
   set DisplayForms.DirName remname ''
   set DisplayForms.DFSDirName remname ''
   set DisplayForms.NullFormDirName remname ''
   set DisplayForms.WdgRow remname ''
   set DisplayForms.WdgCol remname ''
   set DisplayForms.HMargin remname ''
   set DisplayForms.VMargin remname ''
   set DisplayForms.WdgHMargin remname ''
   set DisplayForms.WdgVMargin remname ''
   set DisplayForms.WdgWdth remname ''
   set DisplayForms.WdgHght remname ''
   set DisplayForms.Wdth remname ''
   set DisplayForms.Hght remname ''
   set DisplayForms.WdgZOrder remname ''
   set DisplayForms.ObjectKey remname ''
   set DisplayForms.NullObjectKey remname ''
   set DisplayForms.ObjectType remname ''

   set DDDescriptions.DDOwnerName remname ''
   set DDDescriptions.DDObjectName remname ''
   set DDDescriptions.DirName remname ''
   set DDDescriptions.DDObjectType remname ''
   set DDDescriptions.DDObjectDesc remname ''
   set DDDescriptions.ObjectKey remname ''

   set Users.UserName remname ''
   set Users.UserId remname ''
   set Users.GroupId remname ''

   set Constants.ConstName remname ''
   set Constants.ConstValue remname ''
   set Constants.ConstType remname ''
   set Constants.DirName remname ''
   set Constants.UniqueConstDirName remname ''
   set Constants.ObjectKey remname ''
   set Constants.NullObjectKey remname ''
   set Constants.ObjectType remname ''

   set NamedSets.SetName remname ''
   set NamedSets.SetSpec remname ''
   set NamedSets.AvgSize remname ''
   set NamedSets.DirName remname ''
   set NamedSets.UniqueSetDirName remname ''
   set NamedSets.ObjectKey remname ''
   set NamedSets.NullObjectKey remname ''
   set NamedSets.ObjectType remname ''

   set Windows.WinName remname ''
   set Windows.WNum remname ''
   set Windows.Row remname ''
   set Windows.Col remname ''
   set Windows.Wdth remname ''
   set Windows.Hght remname ''
   set Windows.DM remname ''
   set Windows.BC remname ''
   set Windows.FC remname ''
   set Windows.Fill remname ''
   set Windows.Bord remname ''
   set Windows.BordDM remname ''
   set Windows.BordBC remname ''
   set Windows.BordFC remname ''
   set Windows.BordDefault remname ''
   set Windows.DirName remname ''
   set Windows.UniqueWinDirName remname ''
   set Windows.ObjectKey remname ''
   set Windows.NullObjectKey remname ''
   set Windows.ObjectType remname ''
   set Windows.WinType remname ''
   set Windows.WdgBorderStyle remname ''
   set Windows.WdgCaption remname ''
   set Windows.WdgResize remname ''
   set Windows.WdgMinimize remname ''
   set Windows.WdgMaximize remname ''
   set Windows.WdgSysMenu remname ''
   set Windows.WdgPSWdth remname ''
   set Windows.WdgPSHght remname ''
   set Windows.WdgPSRow remname ''
   set Windows.WdgPSCol remname ''
   set Windows.WdgModal remname ''
   set Windows.WdgScrollBars remname ''
   set Windows.WdgClass remname ''
   set Windows.WdgSubClass remname ''
   set Windows.WdgRow remname ''
   set Windows.WdgCol remname ''
   set Windows.WdgWdth remname ''
   set Windows.WdgHght remname ''
   set Windows.WdgFontSize remname ''
   set Windows.WinOwnerName remname ''
   set Windows.WinOwnerDirName remname ''
   set Windows.WdgAlwaysOnTop remname ''
   set Windows.WdgFCRed remname ''
   set Windows.WdgFCGreen remname ''
   set Windows.WdgFCBlue remname ''
   set Windows.WdgBCRed remname ''
   set Windows.WdgBCGreen remname ''
   set Windows.WdgBCBlue remname ''
   set Windows.WdgLostFocus remname ''
   set Windows.WdgGotFocus remname ''
   set Windows.WdgClosed remname ''
   set Windows.WdgHScrolled remname ''
   set Windows.WdgVScrolled remname ''
   set Windows.WdgMaximized remname ''
   set Windows.WdgMinimized remname ''
   set Windows.WdgMoved remname ''
   set Windows.WdgResized remname ''
   set Windows.WdgRestored remname ''
   set Windows.WdgFont remname ''
   set Windows.WdgLabel remname ''
   set Windows.WdgIcon remname ''
   set Windows.WdgMove remname ''
   set Windows.WdgAutoSize remname ''
   set Windows.HMargin remname ''
   set Windows.VMargin remname ''
   set Windows.WdgHMargin remname ''
   set Windows.WdgVMargin remname ''
   set Windows.WdgAvailable remname ''
   set Windows.WdgTag remname ''
   set Windows.WdgStyle remname ''

   set Menus.MenuName remname ''
   set Menus.DirName remname ''
   set Menus.MenuNum remname ''
   set Menus.WdgTag remname ''
   set Menus.MenuType remname ''
   set Menus.WdgClass remname ''
   set Menus.WdgSubClass remname ''
   set Menus.WdgStyle remname ''
   set Menus.Row remname ''
   set Menus.Col remname ''
   set Menus.WdgRow remname ''
   set Menus.WdgCol remname ''
   set Menus.DM remname ''
   set Menus.FC remname ''
   set Menus.WdgFCRed remname ''
   set Menus.WdgFCGreen remname ''
   set Menus.WdgFCBlue remname ''
   set Menus.BC remname ''
   set Menus.WdgBCRed remname ''
   set Menus.WdgBCGreen remname ''
   set Menus.WdgBCBlue remname ''
   set Menus.WdgBackground remname ''
   set Menus.WdgBorderStyle remname ''
   set Menus.WdgShadowSize remname ''
   set Menus.UniqueMenuDirName remname ''
   set Menus.ObjectKey remname ''
   set Menus.NullObjectKey remname ''
   set Menus.ObjectType remname ''

   set PainterGroups.WinName remname ''
   set PainterGroups.WinDirName remname ''
   set PainterGroups.FDName remname ''
   set PainterGroups.FDDirName remname ''
   set PainterGroups.MenuName remname ''
   set PainterGroups.MenuDirName remname ''
   set PainterGroups.WinDirDirName remname ''
   set PainterGroups.FDDirDirName remname ''
   set PainterGroups.MenuDirDirName remname ''

   set DefinitionsLog.SN remname ''
   set DefinitionsLog.OpCode remname ''
   set DefinitionsLog.ObjectName remname ''
   set DefinitionsLog.ObjectType remname ''
   set DefinitionsLog.OwnerName remname ''
   set DefinitionsLog.OwnerType remname ''
   set DefinitionsLog.DirName remname ''
   set DefinitionsLog.DirId remname ''
   set DefinitionsLog.SysSeqno remname ''
   set DefinitionsLog.OwnerSeqno remname ''
   set DefinitionsLog.TIDBnum remname ''
   set DefinitionsLog.TIDOffset remname ''
   set DefinitionsLog.DescTIDBnum remname ''
   set DefinitionsLog.DescTIDOffset remname ''
   set DefinitionsLog.FileNum remname ''
   set DefinitionsLog.LogDate remname ''
   set DefinitionsLog.LogTime remname ''
   set DefinitionsLog.ZUserId remname ''
   set DefinitionsLog.ZGroupId remname ''
   set DefinitionsLog.DefCommand remname ''

endprocedure

%%========================================================================%
% Upgrading ZIM (Entityset DEFINITIONSLOG)
%%========================================================================%
localprocedure plODv54r10Update( inout vlBreakStatus )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon
%%%%%%%%%   LAT ===> This entityset (DEFINITIONSLOG) is created by ZIMBOOT
%%%%%%%%%            and should be created only when upgrading a database.

IF $EXIST$('ZIM','DefinitionsLog',4) = $False        %%% not yet created?
   plEntAdd( vlBreakStatus, \
      'DefinitionsLog', 'SN', 'longint', '4', \
      '4500', '765', '0087', 'unique', 'yes', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'OpCode', 'int', '2', \
      '4501', '767', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'ObjectName', 'alpha', '18', \
      '4502', '768', 'yes', 'yes', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'ObjectType', 'int', '2', \
      '4503', '769', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'OwnerName', 'alpha', '18', \
      '4504', '770', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'OwnerType', 'int', '2', \
      '4505', '771', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'DirName', 'alpha', '18', \
      '4506', '772', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'DirId', 'int', '2', \
      '4507', '773', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'SysSeqno', 'longint', '4', \
      '4508', '774', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'OwnerSeqno', 'longint', '4', \
      '4509', '775', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'TIDBnum', 'longint', '4', \
      '4510', '776', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'TIDOffset', 'int', '2', \
      '4511', '777', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'DescTIDBnum', 'longint', '4', \
      '4512', '778', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'DescTIDOffset', 'int', '2', \
      '4513', '779', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'FileNum', 'int', '2', \
      '4514', '780', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'LogDate', 'Date', '8', \
      '4515', '781', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'LogTime', 'alpha', '8', \
      '4516', '782', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'ZUserId', 'int', '2', \
      '4517', '783', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'ZGroupId', 'int', '2', \
      '4518', '784', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plFieldAdd( vlBreakStatus, \
      'DefinitionsLog', 'DefCommand', 'varalpha', '320', \
      '4519', '785', 'no', 'no', 'no', '' )
   if( vlBreakStatus = $true )
      set exception break
   endif
ENDIF

   plConstAdd( vlBreakStatus, 'LogDefinitions', 'numeric', '118', '501' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'ServerMode', 'numeric', '3', '502' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   if $exist$( 'ZIM', 'c3DObjectText', 18 ) = $true % only in Alpha revision
@syser
      erase constant c3DObjectColor from ZIM
@syser
      erase constant c3DObjectText from ZIM
      delete all Consts \
         where Consts.ConstName in ('c3DObjectColor', 'c3DObjectText') and \
             Consts.DirName = 'ZIM'
   endif

   plConstAdd( vlBreakStatus, 'c3DObjectColor', 'numeric', '38', '503' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   plConstAdd( vlBreakStatus, 'c3DObjectTextColor', 'numeric', '39', '504' )
   if( vlBreakStatus = $true )
      set exception break
   endif

endprocedure

%%========================================================================%
% Updating ZIM: Adding RemoteProcedures
%%========================================================================%
localprocedure plODv56r10Update( inout vlBreakStatus ) \
  local (vlDefault, vlDefault1)

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   output 'Adding RemoteProcedures'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   plEntAdd( vlBreakStatus, 'RemoteProcedures', 'RemoteProcName', 'varalpha', '128', '10', '820', '0086', 'unique', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'NumInputParams', 'int', '2', '20', '822', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'NumOutputParams', 'int', '2', '30', '823', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'NumResultSets', 'int', '2', '40', '824', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'DocName', 'alpha', '18', '50', '825', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'DirName', 'alpha', '18', '60', '826', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Procedure_Cat', 'alpha', '128', '70', '827', 'no', 'no', 'yes', "''" )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Procedure_Schem', 'alpha', '128', '80', '828', 'no', 'no', 'yes', "''" )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Procedure_Name', 'alpha', '128', '90', '829', 'yes', 'no', 'yes', 'RemoteProcName' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Num_Input_Params', 'int', '2', '100', '830', 'no', 'no', 'yes', 'NumInputParams' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Num_Output_Params', 'int', '2', '110', '831', 'no', 'no', 'yes', 'NumOutputParams' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Num_Result_Sets', 'int', '2', '120', '832', 'no', 'no', 'yes', 'NumResultSets' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Remarks', 'alpha', '254', '130', '833', 'no', 'no', 'yes', "''" )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'Procedure_Type', 'int', '2', '140', '834', 'no', 'no', 'yes', '1' )
   plFieldAdd( vlBreakStatus, 'RemoteProcedures', 'StProcName', 'alpha', '128', '150', '835', 'no', 'no', 'yes', 'RemoteProcName' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   output 'Adding StoredProcedures'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plRoleAdd( vlBreakStatus, 'StoredProcedures', 'RemoteProcedures', '', 839)
   if( vlBreakStatus = $true )
       set exception break
     endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%
%%   output 'Adding ResultSets'
   %%%%%%%%%%%%%%%%%%%%%%%%%%

   plEntAdd( vlBreakStatus, 'ResultSets', 'RemoteProcName', 'varalpha', '128', '10', '840', '0084', 'yes', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultSets', 'SetName', 'alpha', '18', '20', '842', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultSets', 'DirName', 'alpha', '18', '30', '843', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultSets', 'SetSN', 'int', '5', '40', '844', 'unique', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultSets', 'StProcName', 'alpha', '128', '50', '845', 'no', 'no', 'yes', 'RemoteProcName' )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   output 'Adding ResultFields'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plEntAdd( vlBreakStatus, 'ResultFields', 'RemoteProcName', 'varalpha', '128', '10', '850', '0085', 'yes', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'ResultName', 'alpha', '18', '20', '852', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'ResultType', 'int', '2', '30', '853', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Length', 'longint', '4', '40', '854', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Decimals', 'int', '2', '50', '855', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Reqd', 'alpha', '3', '60', '856', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Type', 'alpha', '8', '70', '857', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'OwnerName', 'alpha', '18', '80', '858', 'no', 'no', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'DirName', 'alpha', '18', '90', '859', 'no', 'no', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'SN', 'int', '5', '100', '860', 'no', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'SetSN', 'int', '5', '110', '861', 'yes', 'yes', 'no', '' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Procedure_Cat', 'alpha', '128', '120', '862', 'no', 'no', 'yes', "''" )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Procedure_Schem', 'alpha', '128', '130', '863', 'no', 'no', 'yes', "''" )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Procedure_Name', 'alpha', '128', '140', '864', 'yes', 'no', 'yes', 'RemoteProcName' )
   %plFieldAdd( vlBreakStatus, 'ResultFields', 'Column_Name', 'alpha', '128', '150', '865', 'no', 'no', 'yes', 'ResultName' )
   %plFieldAdd( vlBreakStatus, 'ResultFields', 'Column_Type', 'int', '2', '160', '866', 'no', 'no', 'yes', 'Type' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Data_Type', 'int', '2', '170', '867', 'no', 'no', 'yes', $concat("{", \
        "1 where (Type = 'Alpha' or Type = 'Char') and Length <= 254, ", \
        "12 where Type = 'VarAlpha' or Type = 'VarChar' or Length > 254, ", \
        "5 where Type = 'Int', 4 where Type = 'LongInt', ", \
        "8 where Type = 'Numeric' or Type = 'VastInt', ", \
        "91 where Type = 'Date', 0 }"))
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Zsv_Type', 'int', '2', '180', '868', 'no', 'no', 'yes', $concat("{", \
         "1 where Type = 'Char', ", \
         "2 where Type = 'Alpha', ",\
         "3 where Type = 'VarChar', ", \
         "4 where Type = 'VarAlpha', ", \
         "5 where Type = 'Int', ", \
         "6 where Type = 'LongInt', ", \
         "7 where Type = 'VastInt', ", \
         "8 where Type = 'Date', ", \
         "9 where Type = 'Numeric', 0 }"))
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Type_Name', 'alpha', '128', '190', '869', 'no', 'no', 'yes', 'ResultFields.Type' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Precision', 'longint', '4', '200', '870', 'no', 'no', 'yes', $concat("{", \
        "Length where Type='Alpha' or Type='Char' or Type='VarAlpha' or Type='VarChar', ", \
        "5 where Type='Int', ", \
        "10 where Type='LongInt', ", \
        "15 where Type='VastInt' or Type='Numeric', ", \
        "10 where Type='Date', 0 }"))
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Scale', 'int', '2', '210', '871', 'no', 'no', 'yes', 'Decimals' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Radix', 'int', '2', '220', '872', 'no', 'no', 'yes', '10' )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Nullable', 'int', '2', '230', '873', 'no', 'no', 'yes', "{0 where Reqd = 'Yes', 1 where Reqd = 'No', 2}")
   plFieldAdd( vlBreakStatus, 'ResultFields', 'Remarks', 'alpha', '254', '240', '874', 'no', 'no', 'yes', "''" )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'stProcName', 'alpha', '128', '250', '875', 'no', 'no', 'yes', "RemoteProcName" )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'ColName', 'alpha', '128', '260', '876', 'no', 'no', 'yes', "ResultName" )
   plFieldAdd( vlBreakStatus, 'ResultFields', 'ColType', 'int', '2', '270', '877', 'no', 'no', 'yes', "ResultType" )
   if( vlBreakStatus = $true )
      set exception break
   endif

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   output 'Adding StProcColumns'
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   plRoleAdd( vlBreakStatus, 'StProcColumns', 'ResultFields', '', 879)
     if( vlBreakStatus = $true )
       set exception break
     endif
endprocedure

%%========================================================================%
% Updating ZIM: RightClick
%%========================================================================%
localprocedure plODv60r10Update( inout vlBreakStatus )

on BREAK
   plBreakHandler( $false, vlBreakStatus )
   RETURN
endon


   if $exist$( 'ZIM', 'RightClick', 18 ) = $true
@syserr
      erase constant RightClick from ZIM
      delete all Consts \
         where Consts.ConstName = 'RightClick' and Consts.DirName = 'ZIM'
   endif
   
   plConstAdd( vlBreakStatus, 'RightClick', 'numeric', '70', '508' )

   if( vlBreakStatus = $true )
      set exception break
   endif

endprocedure

%================================ pZimBoot2 =====================================
procedure pZIMBoot2(out vlStatus) \
   local (vlSaveStatus, vlBreakStatus, vlGet, vlSet)

on BREAK
   plBreakHandler( vlSaveStatus, vlBreakStatus )
   let vlStatus = 'BREAK'
   RETURN
endon

on error
   if $lasterrcode <> 4010 % The 'create' command is incomplete
      let vlStatus = 'ERROR'
      SET RESTORE
      RETURN
   endif
endon

%%OUT "STARTING ZIMBOOT2"
%%OUT "=================="
   %% User must be logged in as ZIM or super so objects
   %% belong to correct owner
   if $ZUserid <> 0 or $ZGroupId <> 0
      output '*** Error *** You are not logged in as a "super" user (ZIM, super)'
      output '              Object dictionary extensions cannot be done.'
      output '              Login as ZIM or super and try again.'
      let vlStatus = 'ERROR'
      RETURN
   endif

   SET SAVE
   set errors on
   set warnings on
   set membercount off

   let vlSaveStatus = $true \
       vlBreakStatus = $false

   if $DCRevision < "3.50"
      set quoting on
      plODv52r10Update(vlBreakStatus)
      plODv53r10Update(vlBreakStatus)
      plODv54r10Update(vlBreakStatus)
      plODv56r10Update(vlBreakStatus)
      plODv60r10Update(vlBreakStatus)
   endif

   %% Following resets internal variables in Zim so that objects created
   %% with !CREATE (e.g. in testscn.mac) do not become system objects!
   delete Docs where DocName = '_$ztmp$_' and DirName = 'ZIM'
   add 1 Docs let DocName = '_$ztmp$_' FileName ='zimtemp.tmp' DirName = 'ZIM'
   create doc _$ztmp$_ in ZIM
   set output _$ztmp$_
   list all errtrace
   set output terminal
   set errors off
   create window % causes syntax error # 4010 - OK
   set errors on
   set output errtrace
   list all _$ztmp$_
   set output terminal
   erase doc _$ztmp$_ from Zim
   delete 1 Docs where DocName = '_$ztmp$_' and DirName = 'ZIM'
   let vlStatus = 'OK'
   SET RESTORE

endprocedure
%========================== End of Procedure ====================================
