/***********************************************************************

	Copyright (c) ZIM Technologies Internation Inc. 1983-2002

***********************************************************************/

/***********************************************************************

	"zfunc.h" - Header file for user-written functions callable
	from within ZIM.

	ZIM Syntax:  $userfunction (arg1, arg2, ... argn)
	where arg1, arg2, argn are any valid ZIM character or numeric
	expression.

	The user program is invoked in different ways depending
	on the operating system.  (See zfuncqnx.h for QNX, zfuncux.h
	for UNIX/Xenix, and zfuncdos.h for DOS).

	In all cases, the user program is passed a buffer described
	by the structure tUserBuf defined below.  Refer to the
	end of this file for more detailed notes about the contents
	of this buffer.

	$userfunction is intended for use by experienced programmers.
	Zanthe Information will be unable to help debug user programs
	that do not behave as expected.

***********************************************************************/

	/* C data type used to ZIM "vastint"  and "date" values */

typedef double		XFLOAT;
typedef double		XNUMBER;
typedef int		XMNUMBER;
typedef double		XDATE;

	/* C data type used to store ZIM "int"  values */

typedef short		XSNUMBER;

/* tInfVal is a struct for pointing at information values in general. */

typedef struct {
	char 	InfType;		/* the data type of the value */
	char	InfNullFlag;	/* says whether value is NULL */
	short	decimals;		/* decimal places for numeric */
	short 	vlen;			/* length for strings         */
	union {
		XFLOAT		*pCCXF; /* for float and date */
		XNUMBER		*pCCXN; /* for large numbers */
		XMNUMBER	*pCCXM;	/* for medium sized numbers */
		XSNUMBER	*pCCXS;	/* for short numbers */
		char 		*pCCC;	/* for strings			 	  */
		int			CCDisp;	/* for $Userfunc - offset of Data */
		} uval;
	} tInfVal;

/* the data type codes for ZIM */

#define CCCHAR 		1	/* char strings */
#define CCNUMBER 	2 	/* number fields (vastint, a double in C) */
#define CCCNUMBER 	3	/* numbers in character form */
#define CCSNUMBER	4	/* short numbers (usually 16 bits) */
#define CCMNUMBER	5	/* average size numbers (32 bit int in C) */
#define CCFLOAT		6	/* double (used internally in ZIM) */
#define CCDATE		7	/* date stored as double yyyymmdd */
#define CCTIMESTAMP 13  /* ModTime for ents and fields */

#define CCALPHA		0x40 	/* case insensitive char strings flags */
							/* OR'd with CCCHAR */
#define CCVAR		0x20	/* varying flag OR'd with CCCHAR */
#define CCTYPES		(~(CCALPHA | CCVAR))	/* mask to remove ALPHA and VAR flags */

	/* "derived" types */
#define CCCHALPHA	(CCCHAR | CCALPHA)			/* ALPHA data type */
#define CCVARCHAR	(CCCHAR | CCVAR)			/* VARCHAR */
#define CCVARALPHA	(CCCHAR | CCALPHA | CCVAR)	/* varying ALPHA */

/** Values for the Null Flag "InfNullFlag" in a tInfVal **/

#define		NOTNULL			1	/* value is not null */
#define		UNINULL			2	/* value is a unique null */

/** EQINULL reserved for future use **/
#define		EQINULL			3	/* value is null but implicitly the same */
								/* as the one before it in the set */

/*
 *
 *	A struct that describes the format of the buffer passed to a
 *	user's program on each invocation of $USERFUNCTION from ZIM.
 *
 */

	typedef	struct {
#ifdef UNIXV
		int	UBMsgType;					/* message type for UNIX msgops */
#endif
		short	UBResLen;				/* the maximum length of the result (arg1) */
		short	UBNArgs;				/* the number of arguments */
		tInfVal UBInfVal[1];			/* really (UBNargs + 1) of these */
/*
	Space for the result and the value of each argument follows
	the array of tInfVals "UBInfVal".
*/
	} tUserBuf;

/* System V msg types for UNIX/Xenix */

#ifdef UNIXV
#define INMSG	1L		/* in-coming msg */
#define OUTMSG	2L		/* out-going msg type */
#endif

/*
 *	Notes:
 *
 *		1. UBResLen is the maximum length of the result.
 *
 *		2. UBNargs is the number of parameters passed in the call.
 *
 *		3. UBInfVal is an array of information values described
 *		   by the structure "tInfVal" above.
 *		   There will be exactly (UBNArgs+1) of these. The first
 *         describes arg 0 (the result), the second arg1, and so on.
 *
 *		4. All variable length arguments are converted to fixed length
 *		   before you are passed them (i.e. the CCVAR bit in the InfType
 *		   field on the information value is never turned on.
 *
 *		5. The first argument passed to $userfunction is a prototype for
 *		   the result.  The result inherits its data type and maximum length
 *		   (stored in UBResLen) from the first argument.  The default value
 *		   for the result for character data types is a string of 0 length
 *		   (vlen = 0 in the infval) and zero with zero decimal places
 *		   (decimals = 0 in the infval) for numeric data types.  You
 *		   cannot change the data type.  You can change the length of
 *		   the result (vlen) up to the maximum (UBResLen), the null flag
 *		   (InfNullFlag) and the number of decimals (decimals).
 *		   UBResLen bytes are available in the buffer to store the maximum
 *         length of the result.  This buffer is immediately after the
 *         the array of infvals.
 *
 *		6. The rest of the message contains the data for each
 *		   of the information values in order.  The "CCDisp"
 *		   field in the InfVal for each value will be the
 *		   displacement from the end of the array of InfVals
 *		   to the data.
 *
 *		7. The data is not aligned in any way. On machines which
 *		   require alignment of numeric data types, the data
 *		   must be moved to a variable of the appropiate type
 *		   before performing any arithmetic operations.
 *
 *		8. Data values which are null will have the InfNullFlag
 *		   set to UNINULL (defined above), their vlen set to 0,
 *		   with no data in the buffer.
 *
 *	    9. Numeric constants are converted internally to CCFLOAT
 *		   (double) by ZIM and will appear that way in the buffer.
 *		   This is also true of any arithmetic expressions.
 *
 *	   10. UNIX/Xenix users should define the manifest UNIXV
 *		   before including this file in their program.
 *
 */
